# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/03_era5.ipynb (unless otherwise specified).

__all__ = ['get_config', 'send_request', 'fwi_set', 'era5_get_year', 'era5land_get']

# Cell
import cdsapi
from netCDF4 import Dataset
import matplotlib.pyplot as plt
import numpy as np
from .geo import RegionST
from concurrent.futures import ThreadPoolExecutor
from tqdm import tqdm
from functools import partial
from pathlib import Path

# Cell
def get_config(region:RegionST, variables:list, year:int):
    months = region.times[region.times.year==year].strftime('%m').unique().values.tolist()
    days = region.times[region.times.year==year].strftime('%d').unique().values.tolist()
    times = region.times[region.times.year==year].strftime('%H:%M').unique().tolist()
    bbox = [region.bbox.top, region.bbox.left, region.bbox.bottom, region.bbox.right]
    config = {'format': 'netcdf', 'variable': variables, 'year': [str(year)],
              'month': months, 'day': days, 'time': times,
              'area': f'{"/".join([str(s) for s in bbox])}'} # North, West, South, East
    return config

def send_request(product:str, config:dict, filename:str):
    c = cdsapi.Client()
    print('Sending request')
    c.retrieve(product, config, filename)

def fwi_set():
    return ['10m_u_component_of_wind', '10m_v_component_of_wind', '2m_dewpoint_temperature',
            '2m_temperature', 'surface_pressure', 'total_precipitation']

def era5_get_year(year, region, save_path, variables, product):
    config = get_config(region, variables, year)
    filename = save_path/f'{product}_{region.name}_{year}.nc'
    r = send_request(product, config, str(filename))

def era5land_get(region, save_path, variables=fwi_set(), product='reanalysis-era5-land',
                 max_workers=8):
    f = partial(era5_get_year, region=region, save_path=save_path, variables=variables,
                product=product)
    years = region.times.year.unique().values
    with ThreadPoolExecutor(max_workers) as e:
        list(tqdm(e.map(f, years), total=len(years)))


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"geturl": "00_external.ipynb",
         "sync": "00_external.ipynb",
         "USERAGENT": "00_external.ipynb",
         "DESC": "00_external.ipynb",
         "Ladsweb": "01_download.ipynb",
         "read_log": "01_download.ipynb",
         "update_log": "01_download.ipynb",
         "order_status": "01_download.ipynb",
         "download_files": "01_download.ipynb",
         "release_order": "01_download.ipynb",
         "order_manager": "01_download.ipynb",
         "run_all": "01_download.ipynb",
         "geoget_ladsweb": "02_cli.ipynb",
         "geoget_order_manager": "02_cli.ipynb",
         "get_config": "03_era5.ipynb",
         "send_request": "03_era5.ipynb",
         "fwi_set": "03_era5.ipynb",
         "era5_get_year": "03_era5.ipynb",
         "era5land_get": "03_era5.ipynb",
         "country_bounding_boxes": "04_geo.ipynb",
         "dict2json": "04_geo.ipynb",
         "Region": "04_geo.ipynb",
         "RegionST": "04_geo.ipynb",
         "GFS": "05_gfs.ipynb"}

modules = ["external.py",
           "download.py",
           "cli.py",
           "era5.py",
           "geo.py",
           "gfs.py"]

doc_url = "https://mnpinto.github.io/geoget/"

git_url = "https://github.com/mnpinto/geoget/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/02_cli.ipynb (unless otherwise specified).

__all__ = ['geoget_ladsweb', 'geoget_order_manager']

# Cell
from pathlib import Path
from fastscript import call_parse, Param
import warnings
from time import sleep
from .download import *

# Cell
_repChoices=['ALBERS', 'GEO', 'LAMAZ', 'MERCAT', 'PS', 'ROBIN', 'SNSOID', 'TM', 'UTM']

@call_parse
def geoget_ladsweb(
    product:Param("Name of the product", str),
    collection:Param("Collection number", str),
    tstart:Param("Start of serach window yyyy-mm-dd HH:MM:SS", str),
    tend:Param("End of search windo yyyy-mm-dd HH:MM:SS", str),
    bbox:Param("Bounding box in format left bottom right top", list),
    path_save:Param("Path to save the outputs of the request", str),
    bands:Param("List of bands to download", list),
    coordsOrTiles:Param("coordsOrTiles parameter", str, choices=["coords", "tiles"])="coords",
    daynight:Param("Select images for Day, Night or both", str, choices=['D', 'N', 'DNB'])="DNB",
    repName:Param("Reprojection type", str, choices=_repChoices)='GEO',
    repPixSize:Param("Pixel size in units depending on the reprojection type", float)=0.01,
    repResample:Param("Resampling method", str, choices=['bilinear', 'nearest'])='bilinear',
    doMosaic:Param("",str)='False'):

    bbox = [int(s) for s in ''.join(bbox).split(' ')]
    bands = ''.join(bands).split(' ')
    kwargs = {key: value for key, value in locals().items()}
    kwargs['bands'] = bands
    kwargs['bbox'] = bbox
    lads = Ladsweb(**kwargs)
    lads_list = lads.split_times()
    print(f'Splitting request into {len(lads_list)} orders.')
    run_parallel(lads_list, path_save, email, auth)

# Cell
@call_parse
def geoget_order_manager(path_save:Param("Path where log file is saved.", str)):
    return order_manager(path_save)